import React from 'react';
import PropTypes from 'prop-types';
import moment from 'moment';
import { timeUtils, jobUtils, EstimateGauge, LabelPopover } from '6thman-shared';
import { Link } from 'react-router-dom';
import StageProgress from '../StageProgress/StageProgress.jsx';

import './ActiveJobPanel.css';
class ActiveJobPanel extends React.Component {
  constructor(props) {
    super(props);
    this.jobTimer = null;
    this.stageTimer = null;
    this.breakTimer = null;
    this.state = {
      jobTimePassed: 0,
      stageTimePassed: 0,
      breakTimePassed: 0,
      jobTimeRemaining: 0,
      allBreaks: this.props.job.allBreaks
    };
  }

  componentDidMount() {
		// Get the width of #dashboardContent and substract the padding to get proper gaugeWidth
    const colWidth = document.getElementById('dashboardContent').clientWidth;
    this.gaugeWidth = (colWidth / 4) - 40;
    this.gaugeHeight = (this.gaugeWidth * .75);

    const job = this.props.job;
    this.setJobTimer(job.start.time);
    this.setStageTimer(job.currentStage);
    if (job.currentBreak.body.start.time) {
      this.setBreakTimer(job.currentBreak.body.start.time, job.stages);
    }
  }

  componentWillReceiveProps(nextProps) {
    const thisBreak = this.props.job.currentBreak.body;
    const nextBreak = nextProps.job.currentBreak.body;
		// Start the new stage timer
		// TODO: check if nextProps.currentStageId !== this.props.currentStageId before starting timer
		// it doesn't really matter...but doesn't need to be reset if same currentStage
    clearInterval(this.stageTimer);
    this.setStageTimer(nextProps.job.currentStage);
		// Handle if in a break
    if (!thisBreak.start.time && nextBreak.start.time) {
      this.setBreakTimer(nextBreak.start.time, nextProps.job.stages);
    } else if (thisBreak.start.time && !nextBreak.start.time) {
      clearInterval(this.breakTimer);
    }
  }

  setJobTimer(jobStartTime) {
    this.setState({
      jobTimePassed: timeUtils.calcTimePassed(jobStartTime),
      jobTimeRemaining: timeUtils.calcTimeRemaining(jobStartTime, this.props.job.adjusted_estimate)
    });
    this.jobTimer = setInterval(() => {
      this.setState({
        jobTimePassed: timeUtils.calcTimePassed(jobStartTime),
        jobTimeRemaining: timeUtils.calcTimeRemaining(jobStartTime, this.props.job.adjusted_estimate)
      });
    }, 5000);
  }

  setStageTimer(currentStage) {
    this.setState({stageTimePassed: timeUtils.calcStageTimePassed(currentStage)});
    this.stageTimer = setInterval(() => {
      this.setState({stageTimePassed: timeUtils.calcStageTimePassed(currentStage)});
      console.log(`Stage time passed: ${timeUtils.calcStageTimePassed(currentStage)}`);
    }, 10000);
  }

  setBreakTimer(breakStartTime, stages) {
    this.setState({
      breakTimePassed: timeUtils.calcTimePassed(breakStartTime),
      allBreaks: jobUtils.collateBreaks(stages)
    });
    this.breakTimer = setInterval(() => {
      this.setState({
        breakTimePassed: timeUtils.calcTimePassed(breakStartTime),
        allBreaks: jobUtils.collateBreaks(stages)
      });
      console.log('BREAK time passed: ', timeUtils.calcTimePassed(breakStartTime));
    }, 10000);
  }

  componentWillUnmount() {
    clearInterval(this.jobTimer);
    clearInterval(this.stageTimer);
    clearInterval(this.breakTimer);
  }

  handleOnHover(job) {
    this.props.onHover(job);
  }

//<div className="text-right">
//	<span className="text-muted small">Orig: </span>
//	<span className="small">{moment(job.scheduled_time).format('h:mm')}</span>
//	<span className="small"> - </span>
//	<span className="small">{moment(job.scheduled_time).add(job.original_estimate, 'h').format('h:mm')}</span>
//</div>

//<div className="text-center">
//	<span>{job.completion_rate > 0 ? 'Ahead: ' : 'Behind: '}</span>
//	<span>{job.completion_rate}%</span>
//</div>

  render() {
    const job = this.props.job;
    let Incidents;
    if (job.allIncidents.length) {
      Incidents = job.allIncidents.map((inc, i) => (
				<LabelPopover title={inc.stageTitle} type="warning" key={i} popoverText={inc.body}/>
			));
    } else {
      Incidents = <span className="text-success">All good!</span>;
    }
    let AllBreaks;
    if (this.state.allBreaks.breaks.length) {
      AllBreaks = <span><span>{this.state.allBreaks.breaks.length} </span><span className="text-muted small">{timeUtils.numberToTime(this.state.allBreaks.total)} total time</span></span>;
    } else {
      AllBreaks = <span className="text-success">Chugging along!</span>;
    }
    let CurrentBreak = null;
    if (job.currentBreak.body.start.time) {
      CurrentBreak = <span className="text-danger small pull-right animated fadeIn">
										   On Break - {timeUtils.numberToTime(this.state.breakTimePassed)}
										 </span>;
    }
    return (
			<div className="active-job-panel animated fadeIn">
				<div className="panel panel-default" onMouseEnter={() => {this.handleOnHover(job);}}>
					<div className="panel-body">
						<div className="row active-job-cols">
							<div className="col-sm-9 col-info">
								<div className="job-names">
									<span className="h5 customer-name">{job.customer.full_name}</span>
									<span className="h5">&nbsp;|&nbsp;</span>
									<span className="crew-names">{`${job.user_id.full_name}, ${job.crew}`}</span>
								</div>
								<div className="job-info">
									<div>
										<Link to={`/job/${job.ref_id}`} className="btn btn-default btn-sm">#{job.ref_id}</Link>
									</div>
									<div className="text-right">
										<span className="small job-info-time">
											<span className="strong">Scheduled Start:</span> {moment(job.scheduled_time).format('h:mma')}
										</span>
										<span className="small job-info-time">
											<span className="strong">Actual Start:</span> {moment(job.start.time).format('h:mma')}
										</span>
									</div>
									<div className="text-right">
										<span className="small job-info-time">
											<span className="strong">Original Est: </span>{job.original_estimate}
										</span>
										<span className="small job-info-time">
											<span className="strong">Adjusted Est: </span>{job.adjusted_estimate}
										</span>
									</div>
								</div>
								<div className="job-stats">
									<p><span className="strong">Breaks: </span>{AllBreaks}</p>
									<p><span className="strong">Incidents: </span>{Incidents}</p>
								</div>
							</div>
							<div className="col-sm-3 col-gauge">
								<div className="estimate">
									<div className="text-center">
										<span className="h5">{moment(job.start.time).add(job.adjusted_estimate, 'h').format('h:mma')}</span>
									</div>
									<div className="text-center">
										<span className="h6">{this.state.jobTimeRemaining}</span>
									</div>
								</div>
								<EstimateGauge
									max={Math.round(job.original_estimate + 1)} // Add a 1 hour pad to the og_estimate
									value={this.state.jobTimePassed}
									width={this.gaugeWidth}
									height={this.gaugeHeight}
									adjustedEstimate={job.adjusted_estimate}
									originalEstimate={job.original_estimate}
								/>
							</div>
						</div>
						<div className="row">
							<div className="col-sm-12 col-progress">
								<span className="strong">Current Stage: </span>
								<span className="text-primary">{job.currentStage.title} - </span>
								<span className="text-primary">{timeUtils.numberToTime(this.state.stageTimePassed)} </span>
								<span className="small text-muted">Est: </span>
								<span className="small text-muted">
									{
										job.currentStage.adjusted_estimate ? timeUtils.numberToTime(job.currentStage.adjusted_estimate) :
										timeUtils.numberToTime(job.currentStage.original_estimate)
									}
								</span>
								{CurrentBreak}
								<StageProgress job={job} />
							</div>
						</div>
					</div>
				</div>
			</div>
    );
  }

}

ActiveJobPanel.propTypes = {
  job: PropTypes.object.isRequired,
  onHover: PropTypes.func.isRequired
};

export default ActiveJobPanel;
