import React from 'react';
import PropTypes from 'prop-types';
import moment from 'moment';
import Collapse from '../Collapse/Collapse.jsx';
import { Input, timeUtils } from '6thman-shared';

const timeFormat = timeUtils.formats.time;

import './ActivityList.css';
class ActivityList extends React.Component {
  constructor(props) {
    super(props);
    this.renderQuestions = this.renderQuestions.bind(this);
    this.renderBreaks = this.renderBreaks.bind(this);
    this.renderIncidents = this.renderIncidents.bind(this);
  }
	
  renderQuestions(questions) {
    return (
			<div>
				<h5>Questions</h5>
				{questions.map((question, index) => {
  const validation = question.validation ? question.validation : 'error';
  return (
						<div className={`list-group-item field-input has-${validation}`} key={index}>
							<Input
								name={`${index}`}
								type={question.type}
								value={question.answer || ''}
								label={question.label}
								validation={validation}
								help={question.help}
								placeholder={question.placeholder}
								onChange={() => {}}
								onBlur={() => {}}
								disabled={true}
							/>
						</div>
  );
})}
			</div>
    );
  }
	
  renderBreaks(breaks) {
    return (
			<div>
				<h5>Breaks</h5>
				<table className="table table-striped table-hover">
					<thead>
						<tr>
							<th>Start</th>
							<th>End</th>
							<th>Estimated</th>
							<th>Actual</th>
						</tr>
					</thead>
					<tbody>
						{breaks.map((brk, index) => {
  console.log(brk);
  return (
								<tr>
									<td>{moment(brk.start.time).format(timeFormat)}</td>
									<td>{moment(brk.end.time).format(timeFormat)}</td>
									<td>{brk.original_estimate}</td>
									<td>{moment(brk.end.time).diff(brk.start.time, 'hours', true)}</td>
								</tr>
  );
})}
					</tbody>
				</table>
			</div>
    );
  }
	
  renderIncidents(incidents) {
    return (
			<div>
				<h5>Incidents</h5>
				<table className="table table-striped table-hover">
					<thead>
						<tr>
							<th>Time</th>
							<th>Incident</th>
						</tr>
					</thead>
					<tbody>
						{incidents.map((incident, index) => {
  return (
								<tr>
									<td>{moment(incident.start.time).format(timeUtils.formats.time)}</td>
									<td>{incident.body}</td>
								</tr>
  );
})}
					</tbody>
				</table>
			</div>
    );
  }

  render() {
    const Activities = this.props.stages.map((stage, i) => {
      const stageType = stage.content_template_model;
      return (
				<Collapse title={stage.title} icon={'list'} key={i} isOpen={false}>
				
					<p>Start: {stage.start && stage.start.time ? moment(stage.start.time).format(timeFormat) : 'Stage has not started.'}</p>
		  	
		  		{/* Breaks */}
					{stage.breaks && stage.breaks.length > 0 && this.renderBreaks(stage.breaks)}
			  	
			  	{/* Incidents */}
					{stage.incidents && stage.incidents.length > 0 && this.renderIncidents(stage.incidents)}
				  
					{/* Questions */}
					{stageType === 'StageMilestone' && this.renderQuestions(stage.questions)}
				</Collapse>
      );
    });
    return (
			<div className="list-group activity-list">
				{Activities}
			</div>
    );
  }

}

ActivityList.propTypes = {
  stages: PropTypes.array.isRequired
};

export default ActivityList;
