import React from 'react';
import PropTypes from 'prop-types';
import moment from 'moment';
import { Timeline }  from '6thman-shared';

const earlierDate = (dateString1, dateString2) => {
  if (!dateString1) { return dateString2; }
  if (!dateString2) { return dateString1; }
  return new Date(dateString1) < new Date(dateString2) ? dateString1 : dateString2;
};

const laterDate = (dateString1, dateString2) => {
  if (!dateString1) { return dateString2; }
  if (!dateString2) { return dateString1; }
  return new Date(dateString1) > new Date(dateString2) ? dateString1 : dateString2;
};

const getEstimate = (startDate, estimateHours) => {
  return moment(startDate).add(estimateHours, 'hours').toISOString();
};

import './DashboardTimeline.css';
class DashboardTimeline extends React.Component {
//	constructor(props) {
//		super(props);
//	}

  getScale(jobs) {
    const scale =  jobs.reduce((scale, job) => {

      const originalEstimate = job.original_estimate,
        adjustedEstimate = job.adjusted_estimate;

      const scheduledStart = job.scheduled_time,
        actualStart = job.start.time,
        earlierStart = actualStart ? earlierDate(actualStart, scheduledStart) : scheduledStart;

      const scheduledEnd = getEstimate(scheduledStart, originalEstimate),
        actualEnd = job.end.time,
        laterEstimatedEnd = actualStart ?
							getEstimate(actualStart, adjustedEstimate) :
							getEstimate(scheduledStart, originalEstimate),
        laterEnd = actualEnd ?
							laterDate(actualEnd, laterEstimatedEnd) :
							laterDate(scheduledEnd, laterEstimatedEnd);

      scale.start = earlierDate(scale.start, earlierStart);
      scale.end = laterDate(scale.end, laterEnd);

      return scale;
    }, {start: '', end: '', total: ''});
		// Round to nearest hours and create total diff
    scale.start = moment(scale.start).startOf('hour');
    scale.end = moment(scale.end).endOf('hour');
    scale.total = moment(scale.end).diff(scale.start, 'hours');
    return scale;
  }

  getTasks(jobs) {
    return jobs.map((job) => {
      let mappedJob = {
        id: job.ref_id,
        status: job.status,
        label: `${job.ref_id} - ${job.customer.last_name} (${job.user_id.full_name}, ${job.crew})`,
        link: `/job/${job.ref_id}`,
        original: {
          start: job.scheduled_time,
          end: moment(job.scheduled_time).add(job.original_estimate, 'hours').toISOString(),
          duration: job.original_estimate
        },
        adjusted: {}
      };
      if (job.status === 'inactive') {
        mappedJob.adjusted = mappedJob.original;
        return mappedJob;
      } else if (job.status === 'active') {
        let adjusted = {
          start: job.start.time,
          end: moment(job.start.time).add(job.adjusted_estimate, 'hours').toISOString(),
          duration: job.adjusted_estimate
        };
        mappedJob.adjusted = adjusted;
        return mappedJob;
      } else {
        let adjusted = {
          start: job.start.time,
          end: job.end.time,
          duration: moment(job.end.time).diff(job.start.time, 'hours')
        };
        mappedJob.adjusted = adjusted;
        return mappedJob;
      }
    });
  }

  render() {
    const scale = this.getScale(this.props.jobs);
    const tasks = this.getTasks(this.props.jobs);
    return (
			<div id="dashboardTimeline">
				<Timeline tasks={tasks} scale={scale} showOriginal={true}/>
			</div>
    );
  }

}

DashboardTimeline.propTypes = {
  jobs: PropTypes.array.isRequired
};

export default DashboardTimeline;
