import React from 'react';
import PropTypes from 'prop-types';
import { timeUtils } from '6thman-shared';
import moment from 'moment';

const getMessage = (time) => {
  const minuteHour = timeUtils.numberToTime(Math.abs(time));
	//const message = time >= 1 ? `${minuteHour} hours` : `${minuteHour} minutes`
  if (time >= 0) {
    return `${minuteHour} ahead`;
  } else {
    return `${minuteHour} behind`;
  }
};

const getDiff = (time) => {
  return timeUtils.numberToTime(Math.abs(time));
};

import './JobStats.css';
class JobStats extends React.Component {
//	constructor(props) {
//		super(props);
//	}


  getJobStats(job) {
    const stats = {
      start: {diff: 0, text: 'Not Available'},
      end: {diff: 0, text: 'Not Available'},
      estimate: {diff: 0, text: 'Not Available'}
    };
    if (job.start.time) {
      const startDiff = moment(job.scheduled_time).diff(job.start.time, 'hours', true);
      stats.start = {
        diff: getDiff(startDiff),
        text: getMessage(startDiff)
      };
    }
    if (job.end.time) {
      const estimatedEnd = moment(job.start.time).add(job.original_estimate, 'hours', true);
      const endDiff = moment(estimatedEnd).diff(job.end.time, 'hours', true);
      stats.end = {
        diff: getDiff(endDiff),
        text: getMessage(endDiff)
      };
    }
    if (job.start.time && job.end.time) {
      const actualDuration = moment(job.end.time).diff(job.start.time, 'hours', true);
      console.log(job.original_estimate, actualDuration, job.original_estimate - actualDuration);
      const durationDiff = (job.original_estimate - actualDuration);
      stats.estimate = {
        diff: getDiff(durationDiff),
        text: getMessage(durationDiff)
      };
    }
    console.log('Stats: ', stats);
    return stats;
  }

  render() {
    const stats = this.getJobStats(this.props.job);
    return (
			<ul id="jobStats" className="list-group">
				<li className="list-group-item">
					<span className="badge">{stats.start.diff}</span>
					Started: {stats.start.text}
				</li>
				<li className="list-group-item">
					<span className="badge">{stats.end.diff}</span>
					Ended: {stats.end.text}
				</li>
				<li className="list-group-item">
					<span className="badge">{stats.estimate.diff}</span>
					Estimate: {stats.estimate.text}
				</li>
			</ul>
    );
  }

}

JobStats.propTypes = {
  job: PropTypes.object.isRequired
};

export default JobStats;
