import React from 'react';
import PropTypes from 'prop-types';
import moment from 'moment';
import Select from 'react-select';
import { DateRangePicker } from 'react-dates';

// Third party component styles
import 'react-select/dist/react-select.css';
import 'react-dates/lib/css/_datepicker.css';


import './SearchBar.css';
class SearchBar extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      search: {
        text: '',
        range: {startDate: null, endDate: null},
        select: ''
      },
      focusedRangeInput: null
    };
    this.dateRanges = [
			{label: 'Today', value: {startDate: moment().startOf('day'), endDate: moment().endOf('day')}},
      {label: 'Tomorrow', value: {
        startDate: moment().add(1, 'day').startOf('day'),
        endDate: moment().add(1, 'day').endOf('day')
      }},
      {label: 'This Week', value: {
        startDate: moment().startOf('week'),
        endDate: moment().endOf('week')
      }},
      {label: 'This Month', value: {
        startDate: moment().startOf('month'),
        endDate: moment().endOf('month')
      }},
    ];
    this.numCols = [props.text, props.date, props.select].filter(col => col).length;
  }

  handleOnChange(query) {
    const search = Object.assign({}, this.state.search, query);
    this.setState({search: search});
		//console.log('Search Obj: ', search);
    this.props.onChange(search);
  }

  handleSearchTermChange(event) {
    this.handleOnChange({text: event.target.value});
  }

  clearSearchTerm() {
    this.handleOnChange({text: ''});
  }

  handleDateRangeChange(range) {
		// DateRangePicker returns dates at noon 12:00:00,
		// Convert start/end of day;
    range.startDate = range.startDate ? range.startDate.clone().startOf('day') : null;
    range.endDate = range.endDate ? range.endDate.clone().endOf('day') : null;
    this.handleOnChange({range: range});
  }

  handleDateDropdownChange(option) {
    const range = option ? option.value : {startDate: null, endDate: null};
    this.handleOnChange({range: range});
  }

  handleSelectChange(option) {
    const query = option ? option.value : '';
    this.handleOnChange({select: query});
  }

  render() {
    const colClass = `searchBar-col col-${this.numCols}`;
    return (
			<div id="searchBar" className="panel panel-default">
        {this.props.text &&
          <div className={colClass}>
            <input
              className="searchBar-input"
              placeholder="Search"
              value={this.state.search.text}
              onChange={this.handleSearchTermChange.bind(this)}
            />
            <span onClick={this.clearSearchTerm.bind(this)} id="searchTermClear" className="Select-clear-zone">
              <span className="Select-clear">×</span>
            </span>
          </div>
        }
        {this.props.date &&
          <div className={colClass}>
            <Select
              className="dateRange-select"
              placeholder='Date'
              options={this.dateRanges}
              value={{}}
              onChange={this.handleDateDropdownChange.bind(this)}
              clearable={false}
            />
            <DateRangePicker
              showClearDates={true}
              customCloseIcon={<span className="Select-clear">×</span>}
              startDate={this.state.search.range.startDate} // momentPropTypes.momentObj or null,
              endDate={this.state.search.range.endDate} // momentPropTypes.momentObj or null,
              onDatesChange={this.handleDateRangeChange.bind(this)} // PropTypes.func.isRequired,
              focusedInput={this.state.focusedRangeInput} // PropTypes.oneOf([START_DATE, END_DATE]) or null,
              onFocusChange={focusedRangeInput => this.setState({focusedRangeInput})}
              isOutsideRange={() => false}
              minimumNights={0}
            />
          </div>
        }
        {this.props.select &&
          <div className={colClass}>
            <Select
              placeholder={this.props.selectOptions.title}
              options={this.props.selectOptions.options}
              value={this.state.search.select}
              onChange={this.handleSelectChange.bind(this)}
            />
          </div>
        }
			</div>
    );
  }

}

SearchBar.propTypes = {
  onChange: PropTypes.func.isRequired,
  selectOptions: PropTypes.object.isRequired, // {title: '', options: [{value: '', label: ''}]}
  text: PropTypes.bool,
  date: PropTypes.bool,
  select: PropTypes.bool,
};

SearchBar.defaultProps = {
  text: true,
  date: true,
  select: true,
};


export default SearchBar;
