import React from 'react';
import PropTypes from 'prop-types';
//import moment from 'moment';
import { Input } from '6thman-shared';
import Collapse from '../Collapse/Collapse.jsx';

class AdvancedSection extends React.Component {
  constructor(props) {
    super(props);
    this.state = {isOpen: false};
  }
	
  toggle() { this.setState({isOpen: !this.state.isOpen}); }
	
  render() {
    return (
			<div className="advanced-section">
				<div className="text-right text-muted">
					<i onClick={this.toggle.bind(this)} className="material-icons md-16 advanced-toggle" title="Advanced Settings">perm_data_setting</i>
				</div>
				<div className={`advanced-body ${this.state.isOpen ? '' : 'hidden'}`}>
					{this.props.children}
				</div>
			</div>
    );
  }
}

import './StageList.css';
class StageList extends React.Component {
  constructor(props) {
    super(props);
    this.icons = {
      StageMilestone: 'list',
      StageTravel: 'navigation',
      StageWork: 'slow_motion_video',
    };
  }

  handleEditStage(i, inputData) {
    this.props.onChange(i, inputData);
  }

  displayTemplateCode(name, i) {
    let code = this.props.matrixStages[i][name] ? this.props.matrixStages[i][name].code : '';
    return this.props.codes.includes(code) ? code : '';
  }
	
	// This is not currently used bc the UI is not great for limiting to specific time
	// It is OK at limiting to a day, but not necessarily before 3:41pm
  isValidDate(startTime, selectedTime) {
		// selectedTime is a moment Obj
    return selectedTime.isAfter(startTime);
  }

  render() {
    const Stages = this.props.stages.map((stage, i) => {
      let icon = this.icons[stage.content_template_model];
      let Inputs;
      if (stage.content_template_model === 'StageTravel') {
        Inputs = <div>
									 <div className="row">
										 <div className="col-xs-12">
											 <Input
												 name="origin_location"
												 label="Origin"
												 value={stage.origin_location}
												 onChange={this.handleEditStage.bind(this, i)}
												 help={this.displayTemplateCode('origin_location', i)}
											 />
										 </div>
									 </div>
									 <div className="row">
										 <div className="col-xs-12 last-input">
											 <Input
												 name="destination_location"
												 label="Destination"
												 value={stage.destination_location}
												 onChange={this.handleEditStage.bind(this, i)}
												 help={this.displayTemplateCode('destination_location', i)}
											 />
										 </div>
									 </div>
								 </div>;
      } else if (stage.content_template_model === 'StageWork') {
        Inputs = <div className="row">
									 <div className="col-xs-12">
										 <Input
										 	 type="number"
											 name="original_estimate"
											 label="Original Estimate"
											 value={stage.original_estimate}
											 onChange={this.handleEditStage.bind(this, i)}
											 help={this.displayTemplateCode('original_estimate', i)}
										 />
									 </div>
									 <div className="col-xs-12 last-input">
										 <Input
											 name="location"
											 label="Location"
											 value={stage.location}
											 onChange={this.handleEditStage.bind(this, i)}
											 help={this.displayTemplateCode('location', i)}
										 />
									 </div>
								 </div>;
      } else {
        Inputs = <div>
									 <div className="row">
										 <div className="col-xs-12">
											 <Input
												 name="original_estimate"
												 label="Original Estimate"
												 value={stage.original_estimate}
												 onChange={this.handleEditStage.bind(this, i)}
												 help={this.displayTemplateCode('original_estimate', i)}
											 />
										 </div>
									 </div>
								 </div>;
      }
      return (
				<Collapse title={stage.title} icon={icon} key={i} isOpen={true}>
					{Inputs}
					{this.props.advanced && 
						<AdvancedSection>
							<div className="row">
								<div className="col-sm-6">
									<Input
										 name="start.time"
										 label="Start"
										 type="datetime"
										 value={stage.start.time}
										 onChange={this.handleEditStage.bind(this, i)}
										 disabled={i !== 0}
									 />
								</div>
								<div className="col-sm-6">
									<Input
										 name="end.time"
										 label="End"
										 type="datetime"
										 value={stage.end.time}
										 onChange={this.handleEditStage.bind(this, i)}
										 //isValidDate={this.isValidDate.bind(this, stage.start.time)}
									 />
								</div>
							</div>
						</AdvancedSection>
					}
				</Collapse>
      );
    });
    return (
			<div className="list-group stage-list">
				{Stages}
			</div>
    );
  }

}

StageList.propTypes = {
  stages: PropTypes.array.isRequired,
  matrixStages: PropTypes.array.isRequired,
  codes: PropTypes.array.isRequired,
  onChange: PropTypes.func.isRequired,
  advanced: PropTypes.bool
};

export default StageList;
