import React from 'react';
import PropTypes from 'prop-types';
import moment from 'moment';
import Collapse from '../Collapse/Collapse.jsx';
import { Input, timeUtils } from '6thman-shared';

const timeFormat = timeUtils.formats.time;

import './StatsList.css';
class StatsList extends React.Component {
  constructor(props) {
    super(props);
  }

  renderStart = (job) => {
    const { scheduled_time: scheduled, start : { time: actual } } = job;
    return (
      <div>
        <p className="lead">
          Scheduled Start Time: {moment(job.scheduled_time).format(timeFormat)}
        </p>
        <p className="lead">
          Actual Start Time: {moment(job.start.time).format(timeFormat)}
        </p>
        {scheduled > actual &&
          <p className="lead">
            <span className="material-icons text-success">check_box</span>
            <span>Job started </span>
            <span className="text-success" >
              {moment(scheduled).diff(actual, 'minutes')}
            </span>
            <span> minutes ahead of schedule.</span>
          </p>
        }
        {scheduled < actual &&
          <p className="lead">
          <span className="material-icons text-danger">indeterminate_check_box</span>
            <span>Job started </span>
            <span className="text-danger" >
              {moment(actual).diff(scheduled, 'minutes')}
            </span>
            <span> minutes behind schedule.</span>
          </p>
      }
      </div>
    );
  }

  render() {
    const { job, stages } = this.props;
    const threshold = 10;
    const {
      scheduled_time,
      original_estimate,
      adjusted_estimate,
      allBreaks,
      allIncidents
    } = job;
    const total_time = moment(job.end.time).diff(job.start.time, 'hours', true).toFixed(2);
    const estimate_difference = (((original_estimate / adjusted_estimate) * 100) - 100)
      .toFixed();
    const original_difference = (((original_estimate / total_time) * 100) - 100)
      .toFixed();
    const adjusted_difference = (((adjusted_estimate / total_time) * 100) - 100)
      .toFixed();
    console.log({ job });
    return (
			<div className="stats-list">
        <div>
          {scheduled_time > job.start.time &&
            <p className="lead">
              <span className="material-icons text-success">check_box</span>
              <span>Job started ahead of schedule. -  </span>
              <span className="text-success" >
                {moment(scheduled_time).diff(job.start.time, 'minutes')} minutes
              </span>
            </p>
          }
          {scheduled_time < job.start.time &&
            <p className="lead">
              <span className="material-icons text-danger">indeterminate_check_box</span>
              <span>Job started behind schedule. -  </span>
              <span className="text-danger" >
                {moment(job.start.time).diff(scheduled_time, 'minutes')} minutes
              </span>
            </p>
          }
          <p>Scheduled Start Time: {moment(job.scheduled_time).format(timeFormat)} -
              Actual Start Time: {moment(job.start.time).format(timeFormat)}
          </p>
        </div>

        <hr/>

        <div>
          {estimate_difference < threshold &&
            <p className="lead">
              <span className="material-icons text-success">check_box</span>
              <span>The estimate difference was less than 10%. -  </span>
              <span className="text-success" >
                {estimate_difference}%
              </span>
            </p>
          }
          {estimate_difference > threshold &&
            <p className="lead">
              <span className="material-icons text-danger">indeterminate_check_box</span>
              <span>The estimate difference greater than 10%. -  </span>
              <span className="text-danger" >
                {estimate_difference}%
              </span>
            </p>
          }
          <p>Original Estimate: {original_estimate} hrs -
              Adjuested Estimate: {adjusted_estimate} hrs
            </p>
        </div>

        <hr />

        <div>
          {original_difference < threshold &&
            <p className="lead">
              <span className="material-icons text-success">check_box</span>
              <span>The original estimate was accurate witihin 10%. -  </span>
              <span className="text-success" >
                {original_difference}%
              </span>
            </p>
          }
          {original_difference > threshold &&
            <p className="lead">
              <span className="material-icons text-danger">indeterminate_check_box</span>
              <span>The original estimate was off by more than 10%. -  </span>
              <span className="text-danger" >
                {original_difference}%
              </span>
            </p>
          }
          <p>Original Estimate: {original_estimate} hrs -
              Actual move time: {total_time} hrs
            </p>
        </div>

        <hr/>

        <div>
          {adjusted_difference < threshold &&
            <p className="lead">
              <span className="material-icons text-success">check_box</span>
              <span>The adjusted estimate was accurate witihin 10%. -  </span>
              <span className="text-success" >
                {adjusted_difference}%
              </span>
            </p>
          }
          {adjusted_difference > threshold &&
            <p className="lead">
              <span className="material-icons text-danger">indeterminate_check_box</span>
              <span>The adjusted estimate was off by more than 10%. -  </span>
              <span className="text-danger" >
                {adjusted_difference}%
              </span>
            </p>
          }
          <p>Adjusted Estimate: {adjusted_estimate} hrs -
              Actual move time: {total_time} hrs
            </p>
        </div>

        <hr />

        <div>
          {allBreaks.total > 1 &&
            <p className="lead">
              <span className="material-icons text-success">check_box</span>
              <span>The technicians took a break - </span>
              <span className="text-success" >
                {allBreaks.total} breaks
              </span>
            </p>
          }
          {allBreaks.total < 1 &&
            <p className="lead">
              <span className="material-icons text-danger">indeterminate_check_box</span>
              <span>The techinicians did not take a break -  </span>
              <span className="text-danger" >
                {allBreaks.total} breaks
              </span>
            </p>
          }
        </div>

        <hr />

        <div>
          {allIncidents.length < 1 &&
            <p className="lead">
              <span className="material-icons text-success">check_box</span>
              <span>There were no incidents - </span>
              <span className="text-success" >
              {allIncidents.length} incidents
              </span>
            </p>
          }
          {allIncidents.length > 1 &&
            <p className="lead">
              <span className="material-icons text-danger">indeterminate_check_box</span>
              <span>There were incidents -  </span>
              <span className="text-danger" >
              {allIncidents.length} incidents
              </span>
            </p>
          }
        </div>


			</div>
    );
  }

}

StatsList.propTypes = {
  job: PropTypes.object.isRequired
};

export default StatsList;
