import React from 'react';
import ReactDOM from 'react-dom';
import { BrowserRouter, Redirect, Switch, Route } from 'react-router-dom';
import redrover from 'redrover';
import herald from 'herald';
import { immUtils } from '6thman-shared';

// Pages and Components
import Header from './components/Header/Header.jsx';
import Dashboard from './pages/Dashboard/Dashboard.jsx';
import Users from './pages/Users/Users.jsx';
import User from './pages/User/User.jsx';
import Jobs from './pages/Jobs/Jobs.jsx';
import Stats from './pages/Stats/Stats.jsx';
import Job from './pages/Job/Job.jsx';
import JobTypes from './pages/JobTypes/JobTypes.jsx';
import JobType from './pages/JobType/JobType.jsx';
import QuestionGroups from './pages/QuestionGroups/QuestionGroups.jsx';
import QuestionGroup from './pages/QuestionGroup/QuestionGroup.jsx';
import Account from './pages/Account/Account.jsx';

const pages =  [
  {
    path: '/',
    component: () => (<Redirect to="/dashboard"/>)
  },
  {
    path: '/dashboard',
    component: Dashboard
  },
  {
    path: '/users',
    component: Users
  },
  {
    path: '/user',
    component: User
  },
  {
    path: '/user/:id',
    component: User
  },
  {
    path: '/jobs',
    component: Jobs
  },
  {
    path: '/stats',
    component: Stats,
  },
  {
    path: '/job',
    component: Job
  },
  {
    path: '/job/:id',
    component: Job
  },
  {
    path: '/job-types',
    component: JobTypes
  },
  {
    path: '/job-type',
    component: JobType
  },
  {
    path: '/job-type/:id',
    component: JobType
  },
  {
    path: '/question-groups',
    component: QuestionGroups
  },
  {
    path: '/question-group',
    component: QuestionGroup
  },
  {
    path: '/question-group/:id',
    component: QuestionGroup
  },
  {
    path: '/account',
    component: Account
  }
];

import './index.css';
class AppContainer extends React.Component {
  constructor() {
    super();
    this.state = {
      isReady: false, // wait for the appData before rendering child components that need it
      user: {},
      account: {}
    };
    this.AppData = redrover('/api/app/office');
  }

  componentWillMount() {
    this.AppData.get().then((appData) => {
      this.setState({
        appData: appData,
        isReady: true
      });
    }).catch((err) => {
      herald.error(err, 'Error');
    });
  }

  buildRoutes(pages, appData) {

		// Handle any 404, unknown paths
    const NotFound = <Route component={ () => <h1>404: Page Not Found</h1>} key={100} />;

		// Handle all other pages
    const Routes = pages.map((page, i) => (
			<Route
				exact={true}
				path={page.path}
				//component={page.component}
				// The render func allows additional props to be passed to pages
				render={(props) => <page.component {...props} appData={appData} />}
				key={i}
			/>
		));

		// Collate Matches and MissMatch
    return Routes.concat([NotFound]);
  }

  render() {
    if (!this.state.isReady) { return null; }
    const Routes = this.buildRoutes(pages, immUtils.copyObj(this.state.appData));
    return (
      <BrowserRouter basename='/office'>
        <div>
          <Header appData={this.state.appData} />
          <div className="container-fluid">
            <Switch>
            	{Routes}
						</Switch>
          </div>
        </div>
      </BrowserRouter>
    );
  }
}

ReactDOM.render(<AppContainer/>, document.getElementById('root'));
