import React from 'react';
import redrover from 'redrover';
import herald from 'herald';
import { jobUtils, immUtils, socket, LoadingIndicator } from '6thman-shared';
import DashboardProgress from '../../components/DashboardProgress/DashboardProgress.jsx';
import ActiveJobPanel from '../../components/ActiveJobPanel/ActiveJobPanel.jsx';
import ActiveJobsMap from '../../components/ActiveJobsMap/ActiveJobsMap.jsx';
import DashboardTimeline from '../../components/DashboardTimeline/DashboardTimeline.jsx';

// <ActivityLog log={this.state.activity_log}/>

// TODO: Add a button that sets this.state.center back to null. This will re-center the map based on the
// marker cluster

import './Dashboard.css';
class Dashboard extends React.Component {
  constructor(props) {
    super(props);
    this.Jobs = redrover('/api/jobs');
    this.socketCallback = (data) => {
      const job = data.body;
      const currentJobs = immUtils.copyArr(this.state.jobs);
      let newJobs;
			// if is a job scheduled for today
      if (new Date(job.scheduled_time).setHours(0,0,0,0) === new Date().setHours(0,0,0,0)) {
        jobUtils.createStatefulJobs(job, {adjusted: true})
					.then((updatedJob) => {
  if (!currentJobs.length) {
							// This is the first active job
    newJobs = [updatedJob];
  } else {
							// Find the job and update it
    const activeIndex = currentJobs.findIndex(activeJob => updatedJob.ref_id === activeJob.ref_id);
    if (activeIndex !== -1) {
      newJobs = currentJobs.map((mapJob) => {
        return mapJob.ref_id === updatedJob.ref_id ? updatedJob : mapJob;
      });
    } else {
								// Add the new job
      currentJobs.push(updatedJob);
      newJobs = currentJobs;
    }
  }
  this.setState({jobs: newJobs});
});
      }
    };
    this.state = {
      view: 'timeline',
      jobs: [],
      map: {
				// Pass dummy center on first render. Map progromatically auto centers based on markers
        center: {lat: 0, lng: 0}
      },
      isLoading: true
    };
  }

  componentDidMount() {
    this.Jobs.get({today: true}).then((jobs) => {
      if (jobs.length) {
        jobUtils.createStatefulJobs(jobs, {adjusted: true})
					.then((statefulJobs) => {
  statefulJobs = Array.isArray(statefulJobs) ? statefulJobs : [statefulJobs];
  console.table(statefulJobs);
  this.setState({jobs: statefulJobs, isLoading: false});
});
      } else {
        this.setState({isLoading: false});
      }
    }).catch((err) => {
      herald.error(err, 'Error');
      console.log(err);
    });

		// Register the socket listener
    socket.onAny('job', this.socketCallback.bind(this));
  }

	// Remove the socket listener
  componentWillUnmount() {
    socket.removeOnAny('job', this.socketCallback);
  }

  handleActiveJobHover(job) {
    let coords = immUtils.copyObj(job.current_geo.coords);
    let map = immUtils.copyObj(this.state.map);
    map.center = {lat: coords.latitude, lng: coords.longitude};
    this.setState({map: map});
// The below code will prevent the jank of trying to center on the job already centered,
// but it will not re-center on this job if you panned away (rather than hovered on another)
//		if (coords.latitude !== this.state.map.center.lat ||
//			  coords.longitude !== this.state.map.center.lng) {
//			let map = immUtils.copyObj(this.state.map);
//			map.center = {lat: coords.latitude, lng: coords.longitude};
//			this.setState({map: map});
//		}
  }

  handleOnViewClick(view) {
    if (view !== this.state.view) {
			// re-center the map, TODO: this should reset to where it was
			// TODO why does this re-center the map even if fails conditional?
			// console.log('Reset Center: ', view, this.state.view);
      this.setState({
        view: view,
        map: {center: {lat: 0, lng: 0}}
      });
    }
  }

  render() {
    if (this.state.isLoading) { return <LoadingIndicator />; }
    if (!this.state.jobs.length) { return  <div className="text-center"><h4>There are no active jobs</h4></div>;}
    const activeJobs = this.state.jobs.filter((job) => job.status === 'active');
    const View = this.state.view === 'map' ?
					<ActiveJobsMap jobs={activeJobs} center={this.state.map.center} /> :
					<DashboardTimeline jobs={this.state.jobs}/>;
    return (
      <div id="Dashboard">
       	{View}
       	<div id="dashboardContent">
       	<DashboardProgress jobs={this.state.jobs} onViewClick={this.handleOnViewClick.bind(this)}/>
       		{activeJobs.map((job, i) => (<ActiveJobPanel job={job} onHover={this.handleActiveJobHover.bind(this)} key={i}/>))}
				</div>
      </div>
    );
  }
}

export default Dashboard;
