import React from 'react';
import { Redirect } from 'react-router-dom';
import redrover from 'redrover';
import herald from 'herald';
import { immUtils,
				LoadingIndicator,
				Input,
				PanelHeader,
				jobUtils } from '6thman-shared';
import StageList from '../../components/StageList/StageList.jsx';
import NavSidebar from '../../components/NavSidebar/NavSidebar.jsx';
import JobTimeline from '../../components/JobTimeline/JobTimeline.jsx';
import ActivityList from '../../components/ActivityList/ActivityList.jsx';
import StatsList from '../../components/StatsList/StatsList.jsx';

const templateUtils = {
  propIsObj: function(obj, prop) {
		// There are a number of ways to determine if obj, not sure which best
//		return obj.hasOwnProperty.call(obj, prop);
//		return {}.hasOwnProperty.call(obj, prop);
//		return Object.hasOwnProperty.call(obj, prop);
	  return typeof obj[prop] === 'object' && obj[prop] !== null;
  },
  mapObj: function(obj, path, modifier) {
//		console.log('Mapping...', obj);
    let result = {};
    for (let prop in obj) {
      if (this.propIsObj(obj, prop)) {
        result[prop] = modifier(prop, obj[prop], path);
      }
    }
    return result;
  },
  merge: function(firstObj, secondObj, path, modifier) {
//		console.log('Merging...', firstObj, secondObj);
    return this.mapObj(firstObj, path, (prop, firstObjVal, mergedPath) => {
			//console.log('mergedPath Path: ', mergedPath);
      if (prop in secondObj) {
        mergedPath = mergedPath.length ? `${mergedPath}.${prop}` : prop;
        return modifier(firstObjVal, secondObj[prop], mergedPath);
      } else {
        return firstObjVal;
      }
    });
  },
  mergeRecursive: function(firstObj, secondObj, path, modifier) {
		//console.log('Initial Path: ', path);
//		console.log('Recursively Merging...', this);
    if (!modifier && typeof(path) === 'function') {
      modifier = path;
      path = '';
    }
    return this.merge(firstObj, secondObj, path, (firstObjVal, secondObjVal, mergedPath) => {
			//console.log('Merged Path: ', mergedPath);
      if (typeof firstObjVal === 'object' && typeof secondObjVal === 'object') {
        return this.mergeRecursive(firstObjVal, secondObjVal, mergedPath, modifier);
      } else {
        return modifier(firstObjVal, secondObjVal, mergedPath);
      }
    });
  },
  loopRecursive: function(obj, path, callback) {
    if (!callback && typeof(path) === 'function') {
      callback = path;
      path = '';
    }
    for (let prop in obj) {
      let dotPath = path.length ? `${path}.${prop}` : prop;
//			console.log('Path: ', dotPath);
      if (this.propIsObj(obj, prop)) {
        this.loopRecursive(obj[prop], dotPath, callback);
      } else {
        callback(prop, obj[prop], dotPath);
      }
    }
  },
  mapRecursive: function(obj, path, modifier) {
    if (!modifier && typeof(path) === 'function') {
      modifier = path;
      path = '';
    }
    Object.keys(obj).forEach((prop, i) => {
      let dotPath = path.length ? `${path}.${prop}` : prop;
      if (this.propIsObj(obj, prop)) {
        this.mapRecursive(obj[prop], dotPath, modifier);
      } else {
        obj[prop] = modifier(prop, obj[prop], dotPath);
      }
    });
    return obj;
  },
  isTemplateCode: function(value) {
    return value.length && value.substring(0, 2) === '${';
  }
};

import './Job.css';
class Job extends React.Component {
  constructor(props) {
    super(props);
    this.options = {
      templates: [],
      users: []
    };
    this.state = {
      isLoading: true,
      redirect: false,
      currentTab: 'jobDetails',
      matrix: {},
      job: {},
      template: {},
			template_codes: {},
			duplicate: {}
    };
    this.isEditing = props.match.params.id ? true : false;
    this.Job = redrover(`/api${props.match.url}`);
    this.JobDuplicate = redrover(`/api${props.match.url}/duplicate`);
    this.JobOptions = redrover('/api/jobs/options');
    this.navItems = [
			{title: 'Job Details', id: 'jobDetails'},
			{title: 'Customer', id: 'customer'},
			{title: 'Template Codes', id: 'templateCodes'},
			{title: 'Stages', id: 'stages'},
			{title: 'Activity', id: 'activity'},
			{title: 'Stats', id: 'stats'},
			{title: 'Duplicate', id: 'duplicate'},
    ];
  }

  componentDidMount() {
		// Get the options for assigned_user, JobType, etc
    this.JobOptions.get().then((options) => {
			// Store the options on the class to be used later
      this.options = options;
      let template;
			// Get the info for the job being edited
      if (this.isEditing) {
        this.Job.get()
					.then((response) => (jobUtils.createStatefulJobs(response, {adjusted: true})))
					.then((statefulJob) => {
						// The response/job comes with some fields "populated". Coerce those fields back
						// into the proper templated shape
  let job = immUtils.copyObj(statefulJob);
  job.job_template_id = statefulJob.job_template_id._id;
  job.user_id = statefulJob.user_id._id;
						// Create the matrix and merge the job into the template
  template = this.currentTemplate(options.templates, job.job_template_id);
  this.setState({
    job: job,
    matrix: this.buildMatrix(template),
    isLoading: false
  });
})
					.catch((err) => {
  console.log(err);
  herald.error(err, 'Error');
});
      } else {
				// Build the job and matrix from the "default" / first template in options.templates
        template = this.currentTemplate(options.templates, options.templates[0].value);
        this.setState({
          job: this.buildDefaultTemplate(template),
          matrix: this.buildMatrix(template),
          isLoading: false
        });
      }
    }).catch((err) => {
      herald.error(err, 'Error');
      console.log(err);
    });
  }

  handleJobTypeChange(inputData) {
    let template = this.currentTemplate(this.options.templates, inputData.value);
		// TODO: Merge the old stages? Blow them away for now
    let job = immUtils.copyObj(this.state.job);
    const updatedJob = immUtils.update(job, inputData.name, inputData.value);
    updatedJob.stages = template.stages;
    this.setState({
      job: updatedJob,
      matrix: this.buildMatrix(template),
      template_codes: {}
    }, () => {
      console.log('STATE: ', this.state);
    });
  }

  handleChange(inputData) {
    let job = immUtils.copyObj(this.state.job);
    const updatedJob = immUtils.update(job, inputData.name, inputData.value);
    this.setState({job: updatedJob});
  }

  handleDuplicateChange({ name, value }) {
    this.setState({
      duplicate: {
        ...this.state.duplicate,
        [name]: value
      }
    });
  }

  handleDuplicateSubmit() {
		if (!this.state.duplicate.user_id || !this.state.duplicate.job_template_id) {
			herald.error('Please select a user and job type.', 'Error');
			return;
		}
		const jobTypeName = this.options.templates.find(template => template.value === this.state.duplicate.job_template_id).name;
		const data = {
			user_id: this.state.duplicate.user_id,
			job_template_id: jobTypeName
		};
		this.JobDuplicate.post(data)
      .then(duplicated => {
        herald.success('Job duplicated`', 'Success');
      })
      .catch(error => {
        herald.error(error.message, 'Error');
      });
  }

  handleTemplateCodeChange(inputData) {
    let job = immUtils.copyObj(this.state.job);
    let matrix = immUtils.copyObj(this.state.matrix.matrix);
    let template_codes = immUtils.copyObj(this.state.template_codes);
    console.log(template_codes);
    const updatedTemplateCodes = immUtils.update(template_codes, inputData.name, inputData.value);
    //this.setState({template_codes: updatedTemplateCodes});

    templateUtils.loopRecursive(matrix, (prop, value, path) => {
      if (prop === 'code' && value === inputData.name) {
        console.log('Value: ', value, path.substring(0, path.length - 5), prop);

        job = immUtils.update(job, path.substring(0, path.length - 5), inputData.value);
      }
    });
    this.setState({
      job: job,
      template_codes: updatedTemplateCodes
    });
  }

  handleSubmit(event) {
    event.preventDefault();
    console.log('Job: ', this.state.job);
		// TODO, HERE BE DRAGONS
		// Delete currentBreak because it causes errors when modelUtiles.render is called on the DB
		// This only happens when trying to update an active job
    delete this.state.job.currentBreak;
    const method = this.isEditing ? 'put' : 'post';
    this.Job[method](this.state.job).then((response) => {
      const msg = this.isEditing ? 'Updated!' : 'Created!';
      herald.success(`Job ${response.ref_id} ${msg}`, 'Success');
			// If this job was just created, redirect to the new job's url sp it can be edited
      if (!this.isEditing) {
        this.setState({redirect: `/job/${response.ref_id}`});
      }
    }).catch((err) => {
      const validationErr = err.errors;
      const bestErr = validationErr[Object.keys(validationErr)[0]].message || err.message;
      herald.error(bestErr, 'Error');
    });
  }

  handleDelete(event) {
    this.Job.delete().then((response) => {
      herald.success('Job Deleted!', 'Success');
      this.setState({redirect: '/jobs'});
    }).catch((err) => {
      herald.error(err, 'Error');
    });
  }

  currentTemplate(templates, job_template_id) {
    let template = templates.find((template) => (
			template.value === job_template_id
		));
    return template ? template.template : {};
  }

  buildDefaultTemplate(template) {
    let templateCopy = immUtils.copyObj(template);
    return templateUtils.mapRecursive(templateCopy, (prop, value, path) => {
			// The job_template_id is static and does not change from template to job
			// defaultTemplateId === value in this case
      if (path === 'job_template_id') { return this.options.templates[0].value; }
      if (path === 'user_id') { return this.options.users[0].value; }
			// Handle defaults from template (when has a value but value is not template_code)
      if (!templateUtils.isTemplateCode(value)) { return value; }
      return '';
    });
  }

  buildMatrix(template) {
    let matrix = immUtils.copyObj(template);
    let codes = [];
    const mappedMatrix = templateUtils.mapRecursive(matrix, (prop, value, path) => {
      const isTemplateCode = templateUtils.isTemplateCode(value);
      if (isTemplateCode) { codes.push(value); }
      return {
        path: path,
        code: isTemplateCode ? value : ''
      };
    });
		// Get only the template_codes that are repeated
    codes = codes.filter((code, index, self) => (index !== self.indexOf(code)))
								 .filter((code, index, self) => (index === self.indexOf(code)));
    console.log('Built Matrix Codes: ', codes);
    return {matrix: mappedMatrix, codes: codes};
  }

//	buildJob(template, job) {
//		return templateUtils.mergeRecursive(template, job, (prop, value, path) => {
//			// The job_template_id is static and does not change from template to job
//			// defaultTemplateId === value in this case
//			if (path === 'job_template_id') { return defaultTemplateId; }
//			if (path === 'user.id') { return defaultUser.id; }
//			if (path === 'user.name') { return defaultUser.name; }
//			// Handle defaults from template (when has a value but value is not template_code)
//			if (!templateUtils.isTemplateCode(value)) { return value; }
//			return '';
//		});
//	}

  handleEditStage(i, inputData) {
		//console.log(inputData);
    let job = immUtils.copyObj(this.state.job);
    let stages = immUtils.copyArr(job.stages);
    let stage = immUtils.copyObj(stages[i]);
    stage = immUtils.update(stage, inputData.name, inputData.value);
		// If editing start/stop times, handle other stages
    if (inputData.name === 'start.time') {
			// If modifying first stage start, update job start
			// First stage only one with start.time field
      job.start.time = stage.start.time;
    } else if (inputData.name === 'end.time') {
      if (i === stages.length - 1) {
				// If modifying last stage end, update job end
        job.end.time = stage.end.time;
      } else {
				// Update the next stage's start time
        const nextIndex = i + 1;
        let nextStage = immUtils.copyObj(stages[nextIndex]);
        nextStage = immUtils.update(nextStage, 'start.time', stage.end.time);
        stages[nextIndex] = nextStage;
      }
    }
		//console.log(stage, stages[i + 1]);
    stages[i] = stage;
    job.stages = stages;
    this.setState({job: job});
		//console.log(this.state.job);
  }

  displayTemplateCode(code) {
    return this.state.matrix.codes.includes(code) ? code : '';
  }

  switchTabs(tabId) {
    this.setState({currentTab: tabId});
  }

  render() {
    if (this.state.redirect) { return (<Redirect to={this.state.redirect} />); }
    if (this.state.isLoading) { return <LoadingIndicator />; }
    const job = immUtils.copyObj(this.state.job);
    const matrix = immUtils.copyObj(this.state.matrix.matrix);
    let TemplateCodeInputs = this.state.matrix.codes.map((code, index) => (
			<div className="row" key={index}>
				<div className="col-sm-12">
					<Input
						label={code}
						name={code}
						value={this.state.template_codes[code] || ''}
						onChange={this.handleTemplateCodeChange.bind(this)}
					/>
				</div>
			</div>
		));
    return (
			<div id="Job">
				<div className="row">
					<div className="col-sm-2">
						<NavSidebar navItems={this.navItems} onClick={this.switchTabs.bind(this)}>
							<div>
								<button
									id="saveButton"
									className="btn btn-success btn-round"
									onClick={this.handleSubmit.bind(this)}
								>
									<i className="material-icons">save</i>
								</button>
								<span id="deleteButton" className="delete-text" onClick={this.handleDelete.bind(this)}>Delete</span>
							</div>
						</NavSidebar>
					</div>
					<div className="col-sm-5">

						{/* Move Details Panel */}
						<div className="panel panel-default"
							style={{display: this.state.currentTab === 'jobDetails' ? 'block' : 'none'}}>
							<div className="panel-body">
								<PanelHeader title="Job Details" text="Some helper text about this section" />
								<div className="row">
									<div className="col-sm-6">
										<Input
											label="Job Type"
											name="job_template_id"
											type="select"
											value={job.job_template_id}
											onChange={this.handleJobTypeChange.bind(this)}
											options={this.options.templates}
										/>
									</div>
									<div className="col-sm-6">
										<Input
											label="Job #"
											name="ref_id"
											value={job.ref_id}
											onChange={this.handleChange.bind(this)}
											help={this.displayTemplateCode(matrix.ref_id.code)}
										/>
									</div>
								</div>
								<div className="row">
									<div className="col-sm-6">
										<Input
											type="datetime"
											label="Scheduled Time"
											name="scheduled_time"
											value={job.scheduled_time}
											onChange={this.handleChange.bind(this)}
											timeConstraints={{minutes: {step: 15}}} // Set the time picker to 15 min increments
											help={this.displayTemplateCode(matrix.scheduled_time.code)}
										/>
									</div>
									<div className="col-sm-6">
										<Input
											label="Assigned User"
											name="user_id"
											type="select"
											value={job.user_id}
											onChange={this.handleChange.bind(this)}
											options={this.options.users}
											help={this.displayTemplateCode(matrix.user_id.code)}
										/>
									</div>
								</div>
								<div className="row">
									<div className="col-sm-12">
										<Input
											label="Description"
											name="description"
											type="textarea"
											value={job.description}
											onChange={this.handleChange.bind(this)}
											help={this.displayTemplateCode(matrix.crew.code)}
										/>
									</div>
								</div>
								<div className="row">
									<div className="col-sm-12">
										<Input
											label="Crew"
											name="crew"
											value={job.crew}
											onChange={this.handleChange.bind(this)}
											help={this.displayTemplateCode(matrix.crew.code)}
										/>
									</div>
								</div>
							{/* End of panel-body */}
							</div>
						</div>
						{/* Customer Panel */}
						<div className="panel panel-default"
							style={{display: this.state.currentTab === 'customer' ? 'block' : 'none'}}>
							<div className="panel-body">
								<PanelHeader title="Customer" text="" />
								<div className="row">
									<div className="col-sm-6">
										<Input
											label="First Name"
											name="customer.first_name"
											value={job.customer.first_name}
											onChange={this.handleChange.bind(this)}
											help={this.displayTemplateCode(matrix.customer.first_name.code)}
										/>
									</div>
									<div className="col-sm-6">
										<Input
											label="Last Name"
											name="customer.last_name"
											value={job.customer.last_name}
											onChange={this.handleChange.bind(this)}
											help={this.displayTemplateCode(matrix.customer.last_name.code)}
										/>
									</div>
								</div>
								<div className="row">
									<div className="col-sm-8">
										<Input
											label="Email"
											name="customer.email"
											value={job.customer.email}
											onChange={this.handleChange.bind(this)}
											help={this.displayTemplateCode(matrix.customer.email.code)}
										/>
									</div>
									<div className="col-sm-4">
										<Input
											label="Phone"
											name="customer.phone"
											value={job.customer.phone}
											onChange={this.handleChange.bind(this)}
											help={this.displayTemplateCode(matrix.customer.phone.code)}
										/>
									</div>
									<div className="col-sm-4"></div>
									<div className="col-sm-4"></div>
								</div>
							</div>
						</div>
						{/* Template Codes Panel */}
						<div className="panel panel-default"
							style={{display: this.state.currentTab === 'templateCodes' ? 'block' : 'none'}}>
							<div className="panel-body">
								<PanelHeader title="Template Codes" text="" />
								{TemplateCodeInputs}
							</div>
						</div>
						{/* Stages Panel */}
						<div className="panel panel-default"
							style={{display: this.state.currentTab === 'stages' ? 'block' : 'none'}}>
							<div className="panel-body">
								<PanelHeader title="Stages" text="Some helper text about this section" />
								<StageList
									stages={job.stages}
									matrixStages={matrix.stages}
									codes={this.state.matrix.codes}
									onChange={this.handleEditStage.bind(this)}
									advanced={this.isEditing}
								/>
							</div>
						</div>
						{/* Activity Panel */}
						<div className="panel panel-default"
							style={{display: this.state.currentTab === 'activity' ? 'block' : 'none'}}>
							<div className="panel-body">
								<PanelHeader title="Activity" />
								<ActivityList stages={job.stages} />
							</div>
						</div>
						{/* Stats Panel */}
						<div className="panel panel-default"
							style={{ display: this.state.currentTab === 'stats' ? 'block' : 'none' }}>
							<div className="panel-body">
								<PanelHeader title="Stats" />
								{job.start && job.start.time &&
									<StatsList job={job} stages={job.stages} />
								}
							</div>
						</div>

						{/* Duplicate Panel */}
						<div className="panel panel-default"
							style={{ display: this.state.currentTab === 'duplicate' ? 'block' : 'none' }}>
							<div className="panel-body">
								<PanelHeader title="Create a helper job" />
								<div className="row">
									<div className="col-sm-6">
										<Input
											label="Job Type"
											name="job_template_id"
											type="select"
											value={this.state.duplicate.job_template_id}
											onChange={this.handleDuplicateChange.bind(this)}
											options={this.options.templates}
										/>
									</div>
									<div className="col-sm-6">
										<Input
											label="Assigned User"
											name="user_id"
											type="select"
											value={this.state.duplicate.user_id}
                      onChange={this.handleDuplicateChange.bind(this)}
											options={this.options.users}
										/>
									</div>
                  <div className="col-sm-12">
                    <button
                      className="btn btn-default"
                      onClick={this.handleDuplicateSubmit.bind(this)}
                    >
                    Create Helper Job
                    </button>
                  </div>
								</div>
							</div>
						</div>

					</div>
					<div className="col-sm-5">
						{job.stages[0].start && <JobTimeline job={job}/>}
					</div>
				</div>
			</div>
    );
  }
};

export default Job;


	/* Address Details -
	<PanelDivider text="Address" />
	<div className="row">
		<div className="col-sm-8">
			<Input
				label="Address 1"
				name="address.address_1"
				value={job.address.address_1}
				onChange={this.handleChange.bind(this)}
				help={this.displayTemplateCode(matrix.address.address_1.code)}
			/>
		</div>
		<div className="col-sm-4">
			<Input
				label="Address 2"
				name="address.address_2"
				value={job.address.address_2}
				onChange={this.handleChange.bind(this)}
				help={this.displayTemplateCode(matrix.address.address_2.code)}
			/>
		</div>
	</div>
	<div className="row">
		<div className="col-sm-4">
			<Input
				label="City"
				name="address.city"
				value={job.address.city}
				onChange={this.handleChange.bind(this)}
				help={this.displayTemplateCode(matrix.address.city.code)}
			/>
		</div>
		<div className="col-sm-4">
			<Input
				label="State"
				name="address.state"
				value={job.address.state}
				onChange={this.handleChange.bind(this)}
				help={this.displayTemplateCode(matrix.address.state.code)}
			/>
		</div>
		<div className="col-sm-4">
			<Input
				label="Zip Code"
				name="address.zip"
				value={job.address.zip}
				onChange={this.handleChange.bind(this)}
				help={this.displayTemplateCode(matrix.address.zip.code)}
			/>
		</div>
		<div className="col-sm-4">

		</div>
	</div>
	*/
