import React from 'react';
import redrover from 'redrover';
import herald from 'herald';
import { immUtils, LoadingIndicator } from '6thman-shared';
import { Link } from 'react-router-dom';

import './JobTypes.css';
class JobTypes extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      job_types: [],
      searchTerm: '',
      isLoading: true
    };
    this.JobTypes = redrover('/api/job-types');
  }

  componentDidMount() {
    this.JobTypes.get().then((response) => {
      this.setState({
        job_types: response,
        isLoading: false
      });
    }).catch((err) => {
      herald.error(err, 'Error');
    });
  }

  // Search by title
  handleSearch(event) {
    this.setState({searchTerm: event.target.value});
  }

  render() {
    if (this.state.isLoading) { return <LoadingIndicator />; }
    let job_types = immUtils.copyArr(this.state.job_types);
    const searchTerm = this.state.searchTerm.trim().toLowerCase();
    if (this.state.searchTerm.length) {
      job_types = job_types.filter((type) => (type.title.toLowerCase().indexOf(searchTerm) !== -1));
    }
    const Types = job_types.map((type, i) => (
      <Link to={`/job-type/${type.ref_id}`} className="list-group-item" key={type._id}>
        {type.title}
      </Link>
    ));
    return (
			<div className="col-sm-8 col-sm-offset-2">
				<div className="row">
					<div className="col-sm-4">
						<h4>All Job Types</h4>
					</div>
					<div className="col-sm-4">
						<div className="form-group">
							<input
								placeholder="Search"
								className="form-control"
								value={this.state.searchTerm}
								onChange={this.handleSearch.bind(this)}
							/>
						</div>
					</div>
					<div className="col-sm-4 text-right">
						<Link to="/job-type" className="btn btn-success">Add New Job Type</Link>
					</div>
				</div>
				<div className="list-group">
					{Types}
				</div>
      </div>
    );
  }
};

export default JobTypes;
