import React from 'react';
import redrover from 'redrover';
import herald from 'herald';
import { immUtils, timeUtils, LoadingIndicator } from '6thman-shared';
import moment from 'moment';
import { Link } from 'react-router-dom';
import SearchBar from '../../components/SearchBar/SearchBar.jsx';

const timeFormat = timeUtils.formats;

import './Jobs.css';
class Jobs extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      jobs: [],
      isLoading: true,
      search: {
        text: '',
        range: {startDate: null, endDate: null},
        select: ''
      },
      sort: {
        column: '',
        order: false
      }
    };
    this.Jobs = redrover('/api/jobs');
    this.JobOptions = redrover('/api/jobs/options');
    this.columns = [
			{type: 'id', text: 'Job #', sortable: true},
			{type: 'customer', text: 'Customer', sortable: true},
			{type: 'date', text: 'Date', sortable: true},
			{type: 'user_id', text: 'Crew Leader', sortable: true},
			{type: 'job_template_id', text: 'Job Type', sortable: true},
    ];
    this.searchSelectOptions = {title: 'Crew Leader', options: []};
  }

  componentDidMount() {
    Promise.all([this.JobOptions.get(), this.Jobs.get()])
			.then((response) => {
  this.searchSelectOptions.options = response[0].users.map(user => {
    return {value: user.value, label: user.name};
  });
				// Make all scheduled_times moment objects for easier display and sorting
  let jobs = response[1].map((job) => {
    job.scheduled_time = moment(job.scheduled_time);
    return job;
  });
  this.setState({
    jobs: jobs,
    isLoading: false
  });
})
			.catch((err) => {
  herald.error(err, 'Error');
});
  }

  handleSearch(search) {
    this.setState({search: search});
  }

  handleSort(column) {
		// this.state.sort.order: true = asc, false = desc
    if (column.sortable) {
			// Is same column flip order, else is new column and set to asc
      const order = column.type === this.state.sort.column ? !this.state.sort.order : true;
      this.setState({
        sort: {column: column.type, order: order}
      });
    }
  }

  filterJobs(jobs, search) {
    let filteredJobs = immUtils.copyArr(jobs);
    if (search.text.length) {
      const searchTerm = search.text.toLowerCase();
      filteredJobs = filteredJobs.filter((job) => (
				job.ref_id.toString().indexOf(searchTerm) !== -1 ||
				job.customer.full_name.toLowerCase().indexOf(searchTerm) !== -1 ||
				job.customer.email.toLowerCase().indexOf(searchTerm) !== -1 ||
				job.job_template_id.title.toLowerCase().indexOf(searchTerm) !== -1 ||
				job.crew.toLowerCase().indexOf(searchTerm) !== -1
			));
    }
    if (search.range.startDate && search.range.endDate) {
			// All date objects are already moment objs
      filteredJobs = filteredJobs.filter((job) => {
        return job.scheduled_time.isBetween(
					search.range.startDate,
					search.range.endDate
				);
      });
    } if (search.select) {
      filteredJobs = filteredJobs.filter((job) => {
        return job.user_id._id === search.select;
      });
    }
    return filteredJobs;
  }

  sortJobs(jobs, sort) {
		// this.state.sort.order: true = asc, false = desc
    let sortedJobs = immUtils.copyArr(jobs);
    const order = sort.order;
    if (sort.column === 'id') {
      const sortedBy = (a, b) => (parseInt(a.ref_id, 10) - parseInt(b.ref_id, 10));
      order ? sortedJobs.sort(sortedBy) : sortedJobs.sort(sortedBy).reverse();
    } else if (sort.column === 'customer') {
      const sortBy = (a, b) => (a.customer.last_name < b.customer.last_name ? -1 : 1);
      order ? sortedJobs.sort(sortBy) : sortedJobs.sort(sortBy).reverse();
    } else if (sort.column === 'user_id') {
      const sortBy = (a, b) => (a.user_id.full_name < b.user_id.full_name ? -1 : 1);
      order ? sortedJobs.sort(sortBy) : sortedJobs.sort(sortBy).reverse();
    } else if (sort.column === 'job_template_id') {
      const sortBy = (a, b) => (a['job_template_id'].title < b['job_template_id'].title ? -1 : 1);
      order ? sortedJobs.sort(sortBy) : sortedJobs.sort(sortBy).reverse();
    } else if (sort.column === 'date') {
			// Because scheduled_time is a moment obj with moment#valueOf, < > comparisons work
      const sortBy = (a, b) => (a.scheduled_time < b.scheduled_time ? -1 : 1);
      order ? sortedJobs.sort(sortBy) : sortedJobs.sort(sortBy).reverse();
    }
    return sortedJobs;
  }

  render() {
    if (this.state.isLoading) { return <LoadingIndicator/>; }
    let filteredJobs = this.filterJobs(this.state.jobs, this.state.search);
    filteredJobs = this.sortJobs(filteredJobs, this.state.sort);
    const JobColumns = this.columns.map((column, i) => (
			<th
				onClick={() => {this.handleSort(column);}}
				className={this.state.sort.column === column.type ? 'selected' : null}
				key={i}
			>
				{column.text}
				<i className="material-icons">{this.state.sort.order ? 'keyboard_arrow_up' : 'keyboard_arrow_down'}</i>
			</th>
		));
    const JobRows = filteredJobs.map((job, i) => (
			<tr key={i}>
				<td><Link to={`/job/${job.ref_id}`}>{job.ref_id}</Link></td>
				<td><Link to={`/job/${job.ref_id}`}>{job.customer.full_name}</Link></td>
				<td><Link to={`/job/${job.ref_id}`}>{job.scheduled_time.format(timeFormat.date)}</Link></td>
				<td><Link to={`/job/${job.ref_id}`}>{job.user_id.full_name}</Link></td>
				<td><Link to={`/job/${job.ref_id}`}>{job.job_template_id.title}</Link></td>
			</tr>
		));
    return (
			<div id="Jobs" className="col-sm-8 col-sm-offset-2">
        <SearchBar onChange={this.handleSearch.bind(this)} selectOptions={this.searchSelectOptions}/>
				<div className="row">
					<div className="col-sm-12">
						<Link to="/job" id="newJob" className="btn btn-success btn-round"><i className="material-icons">create</i></Link>
						<table className="table table-striped table-hover panel panel-default">
							<thead>
								<tr>
									{JobColumns}
								</tr>
							</thead>
							<tbody>
								{JobRows}
							</tbody>
						</table>
					</div>
				</div>
			</div>
    );
  }
};

export default Jobs;
