import React from 'react';
import redrover from 'redrover';
import herald from 'herald';
import { immUtils, Input, LoadingIndicator, PanelDivider } from '6thman-shared';
import { Redirect } from 'react-router-dom';

import './QuestionGroup.css';
class QuestionGroup extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      questionGroup: {},
      redirect: false,
      isLoading: true
    };
    this.questionTypeOptions = [
			{name: 'checkbox', value: 'checkbox'},
			{name: 'text', value: 'text'},
			{name: 'textarea', value: 'textarea'},
			{name: 'number', value: 'number'},
    ];
    this.blankQuestion = {
      type: 'text',
      label: 'New Question'
    };
    this.isEditing = props.match.params.id ? true : false;
    this.QuestionGroup = redrover(`/api${props.match.url}`);
  }

  componentDidMount() {
    // If there is an id param, get the user to edit
    if (this.isEditing) {
      this.QuestionGroup.get().then((response) => {
        this.setState({
          questionGroup: response,
          isLoading: false
        });
      }).catch((err) => {
        herald.error(err, 'Error');
      });
    } else {
      this.setState({
        isLoading: false,
        questionGroup: {
          title: '',
          questions: [immUtils.copyObj(this.blankQuestion)]
        }
      });
    }
  }

  handleChange(inputData) {
    let questionGroup = immUtils.copyObj(this.state.questionGroup);
    const updatedQuestionGroup = immUtils.update(questionGroup, inputData.name, inputData.value);
    this.setState({questionGroup: updatedQuestionGroup});
  }

  handleQuestionChange(i, inputData) {
    let questionGroup = immUtils.copyObj(this.state.questionGroup);
    let questions = questionGroup.questions;
    console.log(questions[i]);
    questions[i][inputData.name] = inputData.value;
    questionGroup.questions = questions;
    this.setState({questionGroup: questionGroup});
    console.log(this.state.questionGroup);
  }

  handleRemoveQuestion(i, inputData) {
    let questionGroup = immUtils.copyObj(this.state.questionGroup);
    questionGroup.questions = questionGroup.questions.filter((question, index) => (
			i !== index
		));
    this.setState({questionGroup: questionGroup});
  }

  handleAddQuestion() {
    let questionGroup = immUtils.copyObj(this.state.questionGroup);
    console.log('Adding Question: ', questionGroup);
    questionGroup.questions.push(immUtils.copyObj(this.blankQuestion));
    console.log('Added Question: ', questionGroup);
    this.setState({questionGroup: questionGroup});
  }

  handleSubmit(event) {
    event.preventDefault();
    const method = this.isEditing ? 'put' : 'post';
    this.QuestionGroup[method](this.state.questionGroup).then((response) => {
      console.log('Response: ', response);
      const name = `${response.title} `;
      const msg = this.isEditing ? 'Updated!' : 'Created!';
      herald.success(name + msg, 'Success');
      if (!this.isEditing) {
        this.setState({redirect: `/question-group/${response._id}`});
      }
    }).catch((err) => {
      const validationErr = err.errors;
      const bestErr = validationErr[Object.keys(validationErr)[0]].message || err.message;
      herald.error(bestErr, 'Error');
    });
  }

  handleDelete(event) {
    this.QuestionGroup.delete().then((response) => {
      this.setState({redirect: '/question-groups'});
    }).catch((err) => {
      herald.error(err.message, 'Error');
    });
  }

  render() {
    console.log('State: ', this.state);
    if (this.state.redirect) { return (<Redirect to={this.state.redirect} />); }
    if (this.state.isLoading) { return <LoadingIndicator />; }
    let Questions = this.state.questionGroup.questions.map((question, i) => (
			<div key={i}>
				<PanelDivider text={question.label} />
				<div className="row">
					<div className="col-sm-6">
						<Input
							label="Type"
							name="type"
							value={question.type}
							onChange={this.handleQuestionChange.bind(this, i)}
							type="select"
							options={this.questionTypeOptions}
						/>
					</div>
					<div className="col-sm-6 text-right">
						<span className="delete-text" onClick={this.handleRemoveQuestion.bind(this, i)}>Remove Question</span>
					</div>
				</div>
				<div className="row">
					<div className="col-sm-12">
						<Input
							label="Label"
							name="label"
							value={question.label}
							onChange={this.handleQuestionChange.bind(this, i)}
						/>
					</div>
				</div>
			</div>
		));
    return (
			<div id="QuestionGroup">
				<div className="col-sm-4 col-sm-offset-4">
					<div className="panel panel-default">
						<div className="panel-body">
							<form onSubmit={this.handleSubmit.bind(this)}>
								<div className="row">
									<div className="col-sm-8">
										<Input
											label="Title"
											name="title"
											value={this.state.questionGroup.title}
											onChange={this.handleChange.bind(this)}
										/>
									</div>
									<div className="col-sm-4 text-right">
										<button type="button" className="btn btn-sm btn-default" onClick={this.handleAddQuestion.bind(this)}>Add Question</button>
									</div>
								</div>
								{Questions}
								<div className="row">
									<div className="col-sm-6">
										<span className="delete-text" onClick={this.handleDelete.bind(this)}>Delete</span>
									</div>
									<div className="col-sm-6 text-right">
										<button className="btn btn-success" type="submit">Save</button>
									</div>
								</div>
							</form>
						</div>
					</div>
				</div>
			</div>
    );
  }
};

export default QuestionGroup;
