import React from 'react';
import redrover from 'redrover';
import herald from 'herald';
import { immUtils, LoadingIndicator } from '6thman-shared';
import { Link } from 'react-router-dom';

import './QuestionGroups.css';
class QuestionGroups extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      questionGroups: [],
      searchTerm: '',
      isLoading: true
    };
    this.QuestionGroups = redrover('/api/question-groups');
  }

  componentDidMount() {
    this.QuestionGroups.get().then((response) => {
      this.setState({
        questionGroups: response,
        isLoading: false
      });
    }).catch((err) => {
      herald.error(err, 'Error');
    });
  }

  // Search by last name
  handleSearch(event) {
    this.setState({searchTerm: event.target.value});
  }

  render() {
    let questionGroups = immUtils.copyArr(this.state.questionGroups);
    const searchTerm = this.state.searchTerm.trim().toLowerCase();
    if (this.state.searchTerm.length) {
      questionGroups = questionGroups.filter((group) => (
				group.title.toLowerCase().indexOf(searchTerm) !== -1)
			);
    }
    const QuestionGroups = questionGroups.map((group, i) => (
      <Link to={`/question-group/${group._id}`} className="list-group-item" key={group._id}>
        {group.title}
      </Link>
    ));
    if (this.state.isLoading) { return <LoadingIndicator />; }
    return (
			<div id="QuestionGroups" className="col-sm-8 col-sm-offset-2">
				<div className="row">
					<div className="col-sm-4">
						<h4>All Question Groups</h4>
					</div>
					<div className="col-sm-4">
						<div className="form-group">
							<input
								placeholder="Last Name"
								className="form-control"
								value={this.state.searchTerm}
								onChange={this.handleSearch.bind(this)}
							/>
						</div>
					</div>
					<div className="col-sm-4 text-right">
						<Link to="/question-group" className="btn btn-success">Add New Question Group</Link>
					</div>
				</div>
				<div className="list-group">
					{QuestionGroups}
				</div>
      </div>
    );
  }
};

export default QuestionGroups;
