import React from 'react';
import redrover from 'redrover';
import herald from 'herald';
import { LoadingIndicator, jobUtils } from '6thman-shared';
import moment from 'moment';
import { Link } from 'react-router-dom';
import StatsList from '../../components/StatsList/StatsList.jsx';
import { SingleDatePicker } from 'react-dates';

const chunkArray = (array, chunkSize) => {
  return [].concat.apply([],
    array.map(function (elem, i) {
      return i % chunkSize ? [] : [array.slice(i, i + chunkSize)];
    })
  );
};

import './Stats.css';
class Jobs extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      jobs: [],
      isLoading: true,
      date: new moment()
    };
    this.Jobs = redrover('/api/jobs');
  }

  componentDidMount() {
    this.Jobs.get({ today: true })
      .then(res => {
        if (res.length) {
          jobUtils.createStatefulJobs(res, { adjusted: true })
            .then(jobs => {
              this.setState({
                jobs,
                isLoading: false
              });
            });
        } else {
          this.setState({ isLoading: false });
        }
      })
      .catch((err) => {
        herald.error(err, 'Error');
      });
  }

  handleChange(date) {
    this.setState({ date, isLoading: true });
    this.Jobs.get({ date }).then(res => {
      if (res.length) {
        jobUtils.createStatefulJobs(res, { adjusted: true })
          .then(jobs => {
            this.setState({
              jobs: Array.isArray(jobs) ? jobs : [jobs],
              isLoading: false
            });
          });
      } else {
        this.setState({ isLoading: false });
      }
    });
  }


  render() {
    console.log(this.state)
    if (this.state.isLoading) { return <LoadingIndicator />; }
    return (
      <div id="Stats" className="col-sm-8 col-sm-offset-2">
        <div className="row">
          <div className="col-sm-6">
            <p className="lead title" style={{ marginTop: 10 }}>
              Showing stats for <span className="strong">
                {this.state.date.format('dddd, MMMM Do YYYY')}
              </span>
            </p>
          </div>
          <div className="col-sm-6">
            <span className="lead">
              Select a Report Date:
            </span>
            <SingleDatePicker
              date={this.state.date} // momentPropTypes.momentObj or null
              onDateChange={this.handleChange.bind(this)} // PropTypes.func.isRequired
              focused={this.state.focused} // PropTypes.bool
              onFocusChange={({ focused }) => this.setState({ focused })} // PropTypes.func.isRequired
              id="datepicker" // PropTypes.string.isRequired,
              isOutsideRange={(date) => {
                // only show today and past dates
                return date > new Date();
              }}
            />
          </div>
        </div>
        <hr />
        {!this.state.jobs.length &&
          <div className="text-center"><h4>There are no jobs on that date</h4></div>
        }
        {this.state.jobs.length > 0 &&
          <div>
            {/* <div className="row">
              <div className="col-sm-12">
                <div className="panel panel-default">
                  All Jobs
              </div>
              </div>
            </div> */}
            {chunkArray(this.state.jobs, 2).map((chunk, index) => (
              <div className="row" key={index}>
                <div className="col-sm-6">
                  <div className="panel panel-default">
                    <div className="panel-body">
                      <div className="panel-header page-header">
                        <div className="h5">
                          <Link to={`/job/${chunk[0].ref_id}`}>
                            #{chunk[0].ref_id} - {chunk[0].customer.full_name}
                          </Link>
                        </div>
                      </div>
                      <StatsList job={chunk[0]} />
                    </div>
                  </div>
                </div>
                {chunk[1] &&
                  <div className="col-sm-6">
                    <div className="panel panel-default">
                      <div className="panel-body">
                        <div className="panel-header page-header">
                          <div className="h5">
                            <Link to={`/job/${chunk[1].ref_id}`}>
                              #{chunk[1].ref_id} - {chunk[1].customer.full_name}
                            </Link>
                          </div>
                        </div>
                        <StatsList job={chunk[1]} />
                      </div>
                    </div>
                  </div>
                }
              </div>
            ))}
          </div>
        }
      </div>
    );
  }
};

export default Jobs;
