import React from 'react';
import redrover from 'redrover';
import herald from 'herald';
import { immUtils, Input, LoadingIndicator } from '6thman-shared';
import { Redirect } from 'react-router-dom';

import './User.css';
class User extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      user: {
        ref_id: '',
        first_name: '',
        last_name: '',
        email: '',
        phone: '',
        roles: []
      },
      isLoading: true,
      redirect: false
    };
    this.isEditing = props.match.params.id ? true : false;
    this.User = redrover(`/api${props.match.url}`)
  }

  componentDidMount() {
    // If there is an id param, get the user to edit
    if (this.isEditing) {
      this.User.get().then((response) => {
        this.setState({
          user: response,
          isLoading: false
        });
      }).catch((err) => {
        herald.error(err.message, 'Error');
      });
    } else {
      this.setState({isLoading: false});
    }
  }

  handleChange(inputData) {
    let user = immUtils.copyObj(this.state.user);
    if (inputData.name === 'phone') {
      let mask = inputData.value.replace(/\D/g, '').match(/(\d{0,3})(\d{0,3})(\d{0,4})/);
  		inputData.value = !mask[2] ? mask[1] : '(' + mask[1] + ') ' + mask[2] + (mask[3] ? '-' + mask[3] : '');
    }
    user[inputData.name] = inputData.value;
    this.setState({user: user});
  }

  handleRoleChange(inputData) {
		// inputData.value = true means adding to role, false = removing from roles
    let user = immUtils.copyObj(this.state.user);
    let roles = user.roles;
		// We do some checking here that may not actaully be needed but it is safer...
		// inputData.value ? roles.push(inputData.name) : roles.splice(roles.indexOf(inputData.name), 1);
    if (inputData.value && roles.indexOf(inputData.name) === -1) {
      roles.push(inputData.name);
    } else if (roles.indexOf(inputData.name) !== -1) {
      roles.splice(roles.indexOf(inputData.name), 1);
    }
    user.roles = roles;
    this.setState({user: user});
  }

  handleSubmit(event) {
    event.preventDefault();
    const method = this.isEditing ? 'put' : 'post';
    this.User[method](this.state.user).then((user) => {
      const msg = this.isEditing ? 'Updated!' : 'Created!';
      herald.success(`${user.full_name} ${msg}`, 'Success');
      if (!this.isEditing) {
        this.setState({redirect: `/user/${user.ref_id}`});
      }
    }).catch((err) => {
      const validationErr = err.errors;
      const bestErr = validationErr[Object.keys(validationErr)[0]].message || err.message;
      herald.error(bestErr, 'Error');
    });
  }

  handleDelete(event) {
    this.User.delete().then((response) => {
      this.setState({redirect: '/users'});
    }).catch((err) => {
      herald.error(err.message, 'Error');
    });
  }

  render() {
    if (this.state.isLoading) { return <LoadingIndicator />; }
    if (this.state.redirect) { return <Redirect to={this.state.redirect} />; }
    return (
			<div id="User">
				<div className="col-sm-4 col-sm-offset-4">
					<div className="panel panel-default">
						<div className="panel-body">
							<form onSubmit={this.handleSubmit.bind(this)}>
								<div className="row">
									<div className="col-sm-6">
										<Input
											label="First Name"
											name="first_name"
											value={this.state.user.first_name}
											onChange={this.handleChange.bind(this)}
										/>
									</div>
									<div className="col-sm-6">
										<Input
											label="Last Name"
											name="last_name"
											value={this.state.user.last_name}
											onChange={this.handleChange.bind(this)}
										/>
									</div>
								</div>
								<Input
									label="Email"
									name="email"
									value={this.state.user.email}
									onChange={this.handleChange.bind(this)}
								/>
								<Input
									label="Phone Number"
									name="phone"
									value={this.state.user.phone}
									onChange={this.handleChange.bind(this)}
								/>
								<Input
									label="Ref ID"
									name="ref_id"
									value={this.state.user.ref_id}
									onChange={this.handleChange.bind(this)}
								/>
								<div className="row">
									<div className="col-sm-4">
										<Input
											label="Roles"
											name="technician"
											type="checkbox"
											placeholder="Technician"
											value={this.state.user.roles.indexOf('technician') !== -1}
											onChange={this.handleRoleChange.bind(this)}
										/>
									</div>
									<div className="col-sm-4 inline-checkbox">
										<Input
											name="manager"
											type="checkbox"
											placeholder="Manager"
											value={this.state.user.roles.indexOf('manager') !== -1}
											onChange={this.handleRoleChange.bind(this)}
										/>
									</div>
									<div className="col-sm-4 inline-checkbox">
										<Input
											name="admin"
											type="checkbox"
											placeholder="Admin"
											value={this.state.user.roles.indexOf('admin') !== -1}
											onChange={this.handleRoleChange.bind(this)}
										/>
									</div>
								</div>
								<div className="row">
									<div className="col-sm-6">
										<span className="delete-text" onClick={this.handleDelete.bind(this)}>Delete</span>
									</div>
									<div className="col-sm-6 text-right">
										<button className="btn btn-success" type="submit">Save</button>
									</div>
								</div>
							</form>
						</div>
					</div>
				</div>
			</div>
    );
  }
};

export default User;
