import React from 'react';
import redrover from 'redrover';
import herald from 'herald';
import { immUtils, LoadingIndicator } from '6thman-shared';
import { Link } from 'react-router-dom';

import './Users.css';
class Users extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      users: [],
      searchTerm: '',
      isLoading: true
    };
    this.Users = redrover('/api/users');
  }

  componentDidMount() {
    this.Users.get().then((response) => {
      this.setState({
        users: response,
        isLoading: false
      });
    }).catch((err) => {
      herald.error(err, 'Error');
    });
  }

  // Search by last name
  handleSearch(event) {
    this.setState({searchTerm: event.target.value});
  }

  render() {
    if (this.state.isLoading) { return <LoadingIndicator />; }
    let users = immUtils.copyArr(this.state.users);
    const searchTerm = this.state.searchTerm.trim().toLowerCase();
    if (this.state.searchTerm.length) {
      users = users.filter((user) => (user.last_name.toLowerCase().indexOf(searchTerm) !== -1));
    }
    const Users = users.map((user, i) => (
      <Link to={`/user/${user.ref_id}`} className="list-group-item" key={user.id}>
        {user.full_name} : {user.email}
      </Link>
    ));
    return (
			<div className="col-sm-8 col-sm-offset-2">
				<div className="row">
					<div className="col-sm-4">
						<h4>All Users</h4>
					</div>
					<div className="col-sm-4">
						<div className="form-group">
							<input
								placeholder="Last Name"
								className="form-control"
								value={this.state.searchTerm}
								onChange={this.handleSearch.bind(this)}
							/>
						</div>
					</div>
					<div className="col-sm-4 text-right">
						<Link to="/user" className="btn btn-success">Add New User</Link>
					</div>
				</div>
				<div className="list-group">
					{Users}
				</div>
      </div>
    );
  }
};

export default Users;
